/* morla - Copyright (C) 2005-2006 bakunin - Andrea Marchesini 
 *                                     <bakunin@somasuite.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "editor.h"
#include "morla.h"

static void init_read_config (gchar * file);
static void init_create (void);
static void init_graph (struct graph_data_t *graph, nxml_data_t * data,
			gchar * file);
static void init_graph_reset (void);
static void init_color_reset (void);
static gchar *init_browser (void);
static gchar *init_viewer (void);
static void init_morla_save (void);
static void init_template (void);

#ifndef WIN32
static gboolean init_stat (gchar * sw);
#else
# include <shlwapi.h>
#endif

/* Funzione di inizializzazione di morla: */
void
init (void)
{
  gchar *file;

  /* Mostro lo splash: */
  splash_show ();
  splash_set_text_with_effect (PACKAGE " " VERSION);
  splash_sleep (1);

  /* Setto i valori di default: */
  init_graph_reset ();
  init_color_reset ();

  /* Cerco il file di configurazione: */
  file =
    g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), "." PACKAGE,
		  "config.xml", NULL);

  /* Se il file non esiste, allora e' la prima volta che morla viene
   * avviato e lancio la procedura di creazione config: */
  if (g_file_test (file, G_FILE_TEST_EXISTS) == FALSE)
    init_create ();

  /* Altrimenti leggo la config: */
  else
    {
      /* Inizializzo i template: */
      init_template ();

      /* Leggo la configurazione: */
      init_read_config (file);
    }

  /* Libero memoria e chiudo lo splash: */
  g_free (file);
  splash_hide ();
}

/* Questa funzione legge il file di configurazione passato come argomento: */
static void
init_read_config (gchar * file)
{
  nxml_t *nxml;
  nxml_data_t *data;
  nxml_attr_t *attr;
  struct rdfs_t *rdfs;
  gchar *last;
  gchar *uri;

  /* Setto il titolo nello splash */
  splash_set_text (_("Reading the config file"));

  /* Uso le libnxml che ho fatto io e funzionano tanto bene: */
  if (nxml_new (&nxml) != NXML_OK)
    fatal (_("Error memory!"));

  if (nxml_parse_file (nxml, file) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (nxml_root_element (nxml, &data) != NXML_OK || !data)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (strcmp (data->value, PACKAGE))
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  /* Per ogni elemento del file di configurazione: */
  data = data->children;
  while (data)
    {
      /* Se e' last_id lo aggiorno: */
      if (!strcmp (data->value, "last_id"))
	{
	  nxml_get_string (data, &last);

	  if (!last)
	    last_id = 0;
	  else
	    {
	      last_id = atoi (last);
	      g_free (last);
	    }
	}

      /* Browser: */
      else if (!strcmp (data->value, "browser") && !browser_default)
	nxml_get_string (data, &browser_default);

      /* Viewer: */
      else if (!strcmp (data->value, "viewer") && !viewer_default)
	nxml_get_string (data, &viewer_default);

      /* Normal_bg: */
      else if (!strcmp (data->value, "normal_bg") && !normal_bg_color)
	nxml_get_string (data, &normal_bg_color);

      /* Prelight_bg: */
      else if (!strcmp (data->value, "prelight_bg") && !prelight_bg_color)
	nxml_get_string (data, &prelight_bg_color);

      /* Normal_fg: */
      else if (!strcmp (data->value, "normal_fg") && !normal_fg_color)
	nxml_get_string (data, &normal_fg_color);

      /* Prelight_fg: */
      else if (!strcmp (data->value, "prelight_fg") && !prelight_fg_color)
	nxml_get_string (data, &prelight_fg_color);

      /* I vari RDFS importati: */
      else if (!strcmp (data->value, "rdfs"))
	{
	  GList *list = NULL, *nslist = NULL;

	  /* Alloco un nuovo elemento: */
	  rdfs = (struct rdfs_t *) g_malloc0 (sizeof (struct rdfs_t));

	  /* Cerco l'attributo "resource": */
	  if (nxml_find_attribute (data, "resource", &attr) != NXML_OK
	      || !attr)
	    {
	      g_remove (file);
	      fatal (_("Syntax error in file '%s'"), file);
	    }

	  rdfs->resource = g_strdup (attr->value);

	  /* Cerco l'attributo "prefix": */
	  if (nxml_find_attribute (data, "prefix", &attr) != NXML_OK || !attr)
	    {
	      g_remove (file);
	      fatal (_("Syntax error in file '%s'"), file);
	    }

	  rdfs->prefix = g_strdup (attr->value);

	  /* Prendo la path: */
	  if (nxml_get_string (data, &rdfs->path) != NXML_OK || !rdfs->path)
	    {
	      g_remove (file);
	      fatal (_("Syntax error in file '%s'"), file);
	    }

	  /* Aggiorno lo splash e parso questo rdfs: */
	  splash_set_text (_("Parsing local dump of '%s'"), rdfs->resource);

	  uri = g_filename_to_uri (rdfs->path, NULL, NULL);

	  if (!uri || !(list = rdf_parse (uri, FALSE, NULL, NULL)))
	    {
	      g_remove (rdfs->path);
	      g_remove (file);

	      if (uri)
		g_free (uri);

	      fatal (_("Syntax error in file '%s'"), rdfs->path);
	    }

	  g_free (uri);

	  /* Parso il namespace: */
	  if (!(nslist = namespace_parse (list)))
	    {
	      g_remove (rdfs->path);
	      g_remove (file);
	      fatal (_("Syntax error in file '%s'"), rdfs->path);
	    }

	  /* Aggiungo i dati e inserisco in lista rdfs_list */
	  rdfs->data = list;
	  rdfs->nsdata = nslist;

	  rdfs_list = g_list_append (rdfs_list, rdfs);
	}

      /* Parso i colori dei blank_node nei grafo: */
      else if (!strcmp (data->value, "graph_blank_node"))
	init_graph (&graph_blank_node, data, file);

      /* Parso i colori delle risorse nei grafo: */
      else if (!strcmp (data->value, "graph_resource_node"))
	init_graph (&graph_resource_node, data, file);

      /* Parso i colori dei letterali nei grafo: */
      else if (!strcmp (data->value, "graph_literal_node"))
	init_graph (&graph_literal_node, data, file);

      /* Parso i colori degli archi nei grafo: */
      else if (!strcmp (data->value, "graph_edge"))
	init_graph (&graph_edge, data, file);

      /* Il formato dei grafi: */
      else if (!strcmp (data->value, "graph_format"))
	{
	  nxml_get_string (data, &graph_format);

	  if (!graph_format)
	    {
	      g_remove (file);
	      fatal (_("Syntax error in file '%s'"), file);
	    }
	}

      /* Il livello di undo */
      else if (!strcmp (data->value, "undo_max_value"))
	{
	  gchar *t;

	  nxml_get_string (data, &t);
	  undo_max_value = atoi (t);
	  g_free (t);
	}

      /* Il livello di history */
      else if (!strcmp (data->value, "last_document_value"))
	{
	  gchar *t;

	  nxml_get_string (data, &t);
	  last_document_value = atoi (t);
	  g_free (t);
	}

      /* l'URI per gli aggiornamenti: */
      else if (!strcmp (data->value, "update_url"))
	nxml_get_string (data, &update_url);

      /* l'URI per la documentazione: */
      else if (!strcmp (data->value, "help_url"))
	nxml_get_string (data, &help_url);

      /* Devo mostrare il popup per gli aggiornamenti: */
      else if (!strcmp (data->value, "update_show"))
	{
	  gchar *t;

	  nxml_get_string (data, &t);
	  update_show = atoi (t);
	  g_free (t);
	}

      /* Devo inserire l'estensione di defualt: */
      else if (!strcmp (data->value, "automatic_extensions"))
	{
	  gchar *t;

	  nxml_get_string (data, &t);
	  automatic_extensions = atoi (t) == 1 ? TRUE : FALSE;
	  g_free (t);
	}

      /* Cosi' per ogni nodo dell'XML: */
      data = data->next;
    }

  /* Gli ultimi controlli: */
  if (!browser_default)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (!viewer_default)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (!update_url)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (!help_url)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  /* Certi li setto io di default: */
  if (!undo_max_value)
    undo_max_value = UNDO_MAX_VALUE;

  if (!last_document_value)
    last_document_value = LAST_DOCUMENT_VALUE;

  if (!update_show)
    update_show = 1;

  /* Libero libnxml */
  nxml_free (nxml);
}

/* Questa funzione crea i file di configurazione: */
static void
init_create (void)
{
  gchar *file =
    g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), "." PACKAGE, NULL);

  /* Splash: */
  splash_set_text (_("Creating the config file"));

  /* Creo la directory: */
  if (g_mkdir (file, 0755) && g_file_test (file, G_FILE_TEST_IS_DIR) == FALSE)
    fatal (_("Error creating the directory '%s'!"), file);

  g_free (file);

  /* Creo la directory per gli rdfs importati: */
  file =
    g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), "." PACKAGE, "rdfs",
		  NULL);

  if (g_mkdir (file, 0755) && g_file_test (file, G_FILE_TEST_IS_DIR) == FALSE)
    fatal (_("Error creating the directory '%s'!"), file);

  g_free (file);

  /* Cerco il programmi presenti sul sistema */
  browser_default = init_browser ();
  viewer_default = init_viewer ();

  /* Setto alcuni valori noti: */
  undo_max_value = UNDO_MAX_VALUE;
  last_document_value = LAST_DOCUMENT_VALUE;
  update_url = g_strdup (UPDATE_URL);
  help_url = g_strdup (HELP_URL);

  automatic_extensions = TRUE;

  /* Salvo rdfs di morla di default come importato: */
  init_morla_save ();

  /* Salvo la configurazione appena creata: */
  init_save ();
}

/* Questa funzione salva la configurazione: */
void
init_save (void)
{
  gchar *str;
  gchar *file;
  FILE *fl;
  struct rdfs_t *rdfs;
  GList *list = rdfs_list;

  splash_set_text (_("Saving the config file"));

  /* Apro il file: */
  file =
    g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), "." PACKAGE,
		  "config.xml", NULL);

  if (!(fl = g_fopen (file, "wb")))
    fatal (_("Error creating the file '%s'!"), file);

  g_free (file);

  /* Scrivo dentro l'XML: */
  fprintf (fl, "<?xml version=\"1.0\"?>\n");
  fprintf (fl, "<%s>\n\n", PACKAGE);

  /* Uso markup che e' una funzione per escepare caratteri strani: */
  str = markup (browser_default);
  fprintf (fl, "  <browser>%s</browser>\n", str);
  g_free (str);

  str = markup (viewer_default);
  fprintf (fl, "  <viewer>%s</viewer>\n\n", str);
  g_free (str);

  fprintf (fl, "  <undo_max_value>%d</undo_max_value>\n\n", undo_max_value);
  fprintf (fl, "  <last_document_value>%d</last_document_value>\n\n",
	   last_document_value);

  fprintf (fl, "  <update_show>%d</update_show>\n",
	   update_show >= 0 ? 1 : -1);

  fprintf (fl, "  <automatic_extensions>%d</automatic_extensions>\n",
	   automatic_extensions == TRUE ? 1 : 0);

  str = markup (update_url);
  fprintf (fl, "  <update_url>%s</update_url>\n\n", str);
  g_free (str);

  str = markup (help_url);
  fprintf (fl, "  <help_url>%s</help_url>\n\n", str);
  g_free (str);

  fprintf (fl, "  <last_id>%d</last_id>\n\n", last_id);

  fprintf (fl, "  <normal_bg>%s</normal_bg>\n", normal_bg_color);
  fprintf (fl, "  <prelight_bg>%s</prelight_bg>\n", prelight_bg_color);
  fprintf (fl, "  <normal_fg>%s</normal_fg>\n", normal_fg_color);
  fprintf (fl, "  <prelight_fg>%s</prelight_fg>\n\n", prelight_fg_color);

  fprintf (fl, "  <graph_blank_node\n"
	   "    color=\"%s\"\n"
	   "    fontcolor=\"%s\"\n"
	   "    style=\"%s\"\n"
	   "    fillcolor=\"%s\"\n"
	   "    fontname=\"%s\"\n"
	   "    fontsize=\"%d\"\n"
	   "    shape=\"%s\" />\n", graph_blank_node.color,
	   graph_blank_node.fontcolor, graph_blank_node.style,
	   graph_blank_node.fillcolor, graph_blank_node.fontname,
	   graph_blank_node.fontsize, graph_blank_node.shape);

  fprintf (fl, "  <graph_resource_node\n"
	   "    color=\"%s\"\n"
	   "    fontcolor=\"%s\"\n"
	   "    style=\"%s\"\n"
	   "    fillcolor=\"%s\"\n"
	   "    fontname=\"%s\"\n"
	   "    fontsize=\"%d\"\n"
	   "    shape=\"%s\" />\n", graph_resource_node.color,
	   graph_resource_node.fontcolor, graph_resource_node.style,
	   graph_resource_node.fillcolor, graph_resource_node.fontname,
	   graph_resource_node.fontsize, graph_resource_node.shape);

  fprintf (fl, "  <graph_literal_node\n"
	   "    color=\"%s\"\n"
	   "    fontcolor=\"%s\"\n"
	   "    style=\"%s\"\n"
	   "    fillcolor=\"%s\"\n"
	   "    fontname=\"%s\"\n"
	   "    fontsize=\"%d\"\n"
	   "    shape=\"%s\" />\n", graph_literal_node.color,
	   graph_literal_node.fontcolor, graph_literal_node.style,
	   graph_literal_node.fillcolor, graph_literal_node.fontname,
	   graph_literal_node.fontsize, graph_literal_node.shape);

  fprintf (fl, "  <graph_edge\n"
	   "    color=\"%s\"\n"
	   "    fontcolor=\"%s\"\n"
	   "    style=\"%s\"\n"
	   "    fillcolor=\"%s\"\n"
	   "    fontname=\"%s\"\n"
	   "    fontsize=\"%d\"\n"
	   "    shape=\"%s\" />\n", graph_edge.color, graph_edge.fontcolor,
	   graph_edge.style, graph_edge.fillcolor, graph_edge.fontname,
	   graph_edge.fontsize, graph_edge.shape);

  fprintf (fl, "  <graph_format>%s</graph_format>\n\n", graph_format);

  /* Scrivo tutti gli Rdfs importati: */
  while (list)
    {
      rdfs = list->data;

      fprintf (fl, "  <rdfs");

      if (rdfs->resource)
	{
	  str = markup (rdfs->resource);
	  fprintf (fl, " resource=\"%s\"", str);
	  g_free (str);
	}

      if (rdfs->prefix)
	{
	  str = markup (rdfs->prefix);
	  fprintf (fl, " prefix=\"%s\"", str);
	  g_free (str);
	}

      if (rdfs->path)
	{
	  str = markup (rdfs->path);
	  fprintf (fl, ">%s</rdfs>\n", str);
	  g_free (str);
	}
      else
	fprintf (fl, " />\n");

      list = list->next;
    }

  fprintf (fl, "</%s>\n", PACKAGE);

  /* Chiudo e salvo: */
  fclose (fl);
}

/* Questa funzione parsa gli argomenti di un punto colori del config file
 * cercando tutti gli argomenti essenziali: */
static void
init_graph (struct graph_data_t *graph, nxml_data_t * data, gchar * file)
{
  nxml_attr_t *attr;

  if (nxml_find_attribute (data, "color", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->color)
	g_free (graph->color);
      graph->color = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "fontcolor", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->fontcolor)
	g_free (graph->fontcolor);
      graph->fontcolor = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "style", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->style)
	g_free (graph->style);
      graph->style = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "fillcolor", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->fillcolor)
	g_free (graph->fillcolor);
      graph->fillcolor = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "shape", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->shape)
	g_free (graph->shape);
      graph->shape = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "fontname", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    {
      if (graph->fontname)
	g_free (graph->fontname);
      graph->fontname = g_strdup (attr->value);
    }

  if (nxml_find_attribute (data, "fontsize", &attr) != NXML_OK)
    {
      g_remove (file);
      fatal (_("Syntax error in file '%s'"), file);
    }

  if (attr)
    graph->fontsize = atoi (attr->value);
}

/* Questa funzione setta i valori di default per i colori del grafo: */
static void
init_graph_reset (void)
{
  graph_blank_node.color = g_strdup ("red");
  graph_blank_node.fontcolor = g_strdup ("black");
  graph_blank_node.style = g_strdup ("filled");
  graph_blank_node.fillcolor = g_strdup ("#2266ff");
#ifndef WIN32
  graph_blank_node.fontname = g_strdup ("Sans");
#else
  graph_blank_node.fontname = g_strdup ("Arial");
#endif
  graph_blank_node.shape = g_strdup ("ellipse");
  graph_blank_node.fontsize = 10;

  graph_resource_node.color = g_strdup ("red");
  graph_resource_node.fontcolor = g_strdup ("black");
  graph_resource_node.style = g_strdup ("filled");
  graph_resource_node.fillcolor = g_strdup ("#aabbcc");
#ifndef WIN32
  graph_resource_node.fontname = g_strdup ("Sans");
#else
  graph_resource_node.fontname = g_strdup ("Arial");
#endif
  graph_resource_node.shape = g_strdup ("ellipse");
  graph_resource_node.fontsize = 10;

  graph_literal_node.color = g_strdup ("#ff22aa");
  graph_literal_node.fontcolor = g_strdup ("black");
  graph_literal_node.style = g_strdup ("filled");
  graph_literal_node.fillcolor = g_strdup ("#ccbbaa");
#ifndef WIN32
  graph_literal_node.fontname = g_strdup ("Sans");
#else
  graph_literal_node.fontname = g_strdup ("Arial");
#endif
  graph_literal_node.fontsize = 10;
  graph_literal_node.shape = g_strdup ("box");

  graph_edge.color = g_strdup ("black");
  graph_edge.fontcolor = g_strdup ("black");
  graph_edge.style = g_strdup ("solid");
  graph_edge.fillcolor = g_strdup ("#ccbbaa");
#ifndef WIN32
  graph_edge.fontname = g_strdup ("Sans");
#else
  graph_edge.fontname = g_strdup ("Arial");
#endif
  graph_edge.fontsize = 10;
  graph_edge.shape = g_strdup ("box");

  graph_format = g_strdup ("png");
}

/* Questa funzione setta i colori dei default per morla: */
static void
init_color_reset (void)
{
  normal_bg_color = g_strdup ("#ffffff");
  prelight_bg_color = g_strdup ("#00ffff");
  normal_fg_color = g_strdup ("#10297c");
  prelight_fg_color = g_strdup ("#8c0498");
}

#ifndef WIN32
/* Questa funzione cerca un programma 'sw' usando la variabile di ambiente
 * PATH. Se lo trova, comunica ok: */
static gboolean
init_stat (gchar * sw)
{
  gchar *path;

  if ((path = g_find_program_in_path (sw)))
    {
      g_free (path);
      return 0;
    }

  return 1;
}
#endif

/* Questa funzione cerca i programmi di default per morla win32 version */
static gchar *
init_browser (void)
{
#ifdef WIN32
  GList *list, *tmp;
  GString *ret;
  char szDefault[MAX_PATH];
  DWORD ccDefault = MAX_PATH;
  HRESULT rc = AssocQueryString (0, ASSOCSTR_EXECUTABLE,
				 "http", "open", szDefault, &ccDefault);

  if (!SUCCEEDED (rc) || !(tmp = list = split (szDefault)))
    return g_strdup ("firefox %s");

  ret = g_string_new (NULL);
  g_string_append_printf (ret, "\"%s\"", (gchar *) tmp->data);

  tmp = tmp->next;

  while (tmp)
    {
      if (!strcmp ((gchar *) tmp->data, "%1"))
	g_string_append_printf (ret, " %%s");
      else
	g_string_append_printf (ret, " \"%s\"", (gchar *) tmp->data);

      tmp = tmp->next;
    }

  g_list_foreach (list, (GFunc) g_free, NULL);
  g_list_free (list);

  return g_string_free (ret, FALSE);

#else

  if (!init_stat ("x-www-browser"))
    return g_strdup ("x-www-browser %s");

  if (!init_stat ("firefox"))
    return g_strdup ("firefox %s");

  if (!init_stat ("mozilla"))
    return g_strdup ("mozilla %s");

  if (!init_stat ("dillo"))
    return g_strdup ("dillo %s");

  if (!init_stat ("epiphany"))
    return g_strdup ("epiphany %s");

  if (!init_stat ("konqueror"))
    return g_strdup ("konqueror %s");

  if (!init_stat ("galeon"))
    return g_strdup ("galeon %s");

  return g_strdup ("firefox %s");
#endif
}

/* Questa funzione cerca il viewer: */
static gchar *
init_viewer (void)
{
#ifdef WIN32
  GList *tmp, *list;
  GString *ret;
  char szDefault[MAX_PATH];
  DWORD ccDefault = MAX_PATH;
  HRESULT rc = AssocQueryString (0, ASSOCSTR_EXECUTABLE,
				 ".png", "open", szDefault, &ccDefault);

  if (!SUCCEEDED (rc) || !(tmp = list = split (szDefault)))
    return g_strdup ("firefox %s");

  ret = g_string_new (NULL);
  g_string_append_printf (ret, "\"%s\"", (gchar *) tmp->data);

  tmp = tmp->next;

  while (tmp)
    {
      if (!strcmp ((gchar *) tmp->data, "%1"))
	g_string_append_printf (ret, " %%s");
      else
	g_string_append_printf (ret, " \"%s\"", (gchar *) tmp->data);

      tmp = tmp->next;
    }

  g_list_foreach (list, (GFunc) g_free, NULL);
  g_list_free (list);

  return g_string_free (ret, FALSE);
#else

  if (!init_stat ("gqview"))
    return g_strdup ("gqview -f %s");

  if (!init_stat ("gimp"))
    return g_strdup ("gimp -d -s -f %s");

  if (!init_stat ("kview"))
    return g_strdup ("kview %s");

  return g_strdup ("gqview -f %s");
#endif
}

/* Questa funzione salva l'RDFS di morla: */
static void
init_morla_save (void)
{
  gchar *file;
  int fd;
  int len, done, ret;
  gchar *uri;
  struct rdfs_t *rdfs;
  GList *list;

  splash_set_text (_("Creating the RDFS morla file"));

  /* Cancello il precedente: */
  file = rdfs_file ();
  g_remove (file);

  /* Apro, scrivo, chiudo: */
  if ((fd = open (file, O_WRONLY | O_CREAT, 0644)) < 0)
    return;

  len = strlen (morla_rdfs);
  done = 0;

  while (done < len)
    {
      if ((ret = write (fd, morla_rdfs + done, len - done)) < 1)
	fatal (_("Error writing file '%s'!"), file);

      done += ret;
    }

  close (fd);

  /* Parso il nuovo file: */
  uri = g_filename_to_uri (file, NULL, NULL);

  if (!uri || !(list = rdf_parse (uri, FALSE, NULL, NULL)))
    {
      if (uri)
	g_free (uri);

      g_remove (file);
      return;
    }

  /* Creo la struttura e inserisco: */
  rdfs = (struct rdfs_t *) g_malloc0 (sizeof (struct rdfs_t));

  rdfs->resource = g_strdup (MORLA_NS);
  rdfs->prefix = g_strdup ("morla");	/* Si questo di default... */

  rdfs->data = list;
  rdfs->nsdata = namespace_parse (list);
  rdfs->path = file;

  rdfs_list = g_list_append (rdfs_list, rdfs);

  /* Fatto questo importo i template presenti nell'RDFS di morla: */
  template_import (uri, FALSE, FALSE);
  template_save ();

  g_free (uri);
}

/* Parsa il file di template e lo gestisce: */
static void
init_template (void)
{
  gchar *path;
  gchar *uri;

  splash_set_text (_("Reading the template file"));

  /* Se non esiste esco: */
  path =
    g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), "." PACKAGE,
		  "template.rdf", NULL);
  if (g_file_test (path, G_FILE_TEST_EXISTS) == FALSE)
    {
      g_free (path);
      return;
    }

  /* Importo questo uri: */
  uri = g_filename_to_uri (path, NULL, NULL);

  if (!uri || template_import (uri, FALSE, FALSE))
    {
      splash_hide ();
      dialog_msg (_("Import template error!"));
      g_remove (path);
    }

  if (uri)
    g_free (uri);

  g_free (path);
}

/* EOF */
